<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;

use DB;
use Storage;
use Validator;

class ClientController extends Controller
{
    private $table = "clients";

    public function change_password()
    {
        $validator = Validator::make(request()->all(), [
            "current_password" => "required",
            "new_password" => "required"
        ]);

        if (!$validator->passes() && count($validator->errors()->all()) > 0)
        {
            return response()->json([
                "status" => "error",
                "message" => $validator->errors()->all()[0]
            ]);
        }

        $client = $this->auth_client();
        $current_password = request()->current_password ?? "";
        $new_password = request()->new_password ?? "";

        if (!password_verify($current_password, $client->password))
        {
            return response()->json([
                "status" => "error",
                "message" => "In-correct password."
            ]);
        }

        DB::table("clients")
            ->where("id", "=", $client->id)
            ->update([
                "password" => password_hash($new_password, PASSWORD_DEFAULT),
                "updated_at" => now()->utc()
            ]);

        return response()->json([
            "status" => "success",
            "message" => "Password has been changed."
        ]);
    }

    public function save_profile()
    {
        $validator = Validator::make(request()->all(), [
            "name" => "required"
        ]);

        if (!$validator->passes() && count($validator->errors()->all()) > 0)
        {
            return response()->json([
                "status" => "error",
                "message" => $validator->errors()->all()[0]
            ]);
        }

        $client = $this->auth_client();
        $name = request()->name ?? "";

        DB::table("clients")
            ->where("id", "=", $client->id)
            ->update([
                "name" => $name,
                "updated_at" => now()->utc()
            ]);

        return response()->json([
            "status" => "success",
            "message" => "Profile has been saved."
        ]);
    }

    public function me()
    {
        $client = $this->auth_client();

        return response()->json([
            "status" => "success",
            "message" => "Data has been fetched.",
            "client" => [
                "id" => $client->id,
                "name" => $client->name ?? "",
                "email" => $client->email ?? ""
            ]
        ]);
    }

    public function login()
    {
        $validator = Validator::make(request()->all(), [
            "email" => "required",
            "password" => "required"
        ]);

        if (!$validator->passes() && count($validator->errors()->all()) > 0)
        {
            return response()->json([
                "status" => "error",
                "message" => $validator->errors()->all()[0]
            ]);
        }

        $email = request()->email ?? "";
        $password = request()->password ?? "";

        $client = DB::table($this->table)
            ->where("email", "=", $email)
            ->whereNull("deleted_at")
            ->first();

        if ($client == null)
        {
            return response()->json([
                "status" => "error",
                "message" => "Email does not exist."
            ]);
        }

        if (!password_verify($password, $client->password))
        {
            return response()->json([
                "status" => "error",
                "message" => "In-correct password."
            ]);
        }

        $token = $this->generate_jwt($client->id);

        return response()->json([
            "status" => "success",
            "message" => "Login successfully.",
            "access_token" => $token
        ]);
    }

    public function destroy()
    {
        $validator = Validator::make(request()->all(), [
            "id" => "required"
        ]);

        if (!$validator->passes() && count($validator->errors()->all()) > 0)
        {
            return response()->json([
                "status" => "error",
                "message" => $validator->errors()->all()[0]
            ]);
        }

        $this->admin_auth();
        $id = request()->id ?? 0;

        $client = DB::table($this->table)
            ->where("id", "=", $id)
            ->whereNull("deleted_at")
            ->first();

        if ($client == null)
        {
            return response()->json([
                "status" => "error",
                "message" => "Client not found."
            ]);
        }

        DB::table($this->table)
            ->where("id", "=", $client->id)
            ->update([
                "deleted_at" => now()->utc()
            ]);

        return response()->json([
            "status" => "success",
            "message" => "Client has been deleted."
        ]);
    }

    public function update()
    {
        $validator = Validator::make(request()->all(), [
            "id" => "required",
            "name" => "required"
        ]);

        if (!$validator->passes() && count($validator->errors()->all()) > 0)
        {
            return response()->json([
                "status" => "error",
                "message" => $validator->errors()->all()[0]
            ]);
        }

        $this->admin_auth();
        $id = request()->id ?? 0;
        $name = request()->name ?? "";
        $email = request()->email ?? "";
        $phone = request()->phone ?? "";
        $password = request()->password ?? "";

        $client = DB::table($this->table)
            ->where("id", "=", $id)
            ->whereNull("deleted_at")
            ->first();

        if ($client == null)
        {
            return response()->json([
                "status" => "error",
                "message" => "Client not found."
            ]);
        }

        $client_id = $client->id;

        $exists_by = "";
        $client = DB::table($this->table);
        if (!empty($email))
        {
            $client = $client->where("email", "=", $email)
                ->where("id", "!=", $client_id);
            $exists_by = "email";
        }
        if (!empty($phone))
        {
            $client = $client->where("phone", "=", $phone)
                ->where("id", "!=", $client_id);
            $exists_by = "phone";
        }
        $client = $client->first();

        if ($client != null)
        {
            return response()->json([
                "status" => "error",
                "message" => "Client with same " . $exists_by . " already exists."
            ]);
        }

        $obj = [
            "name" => $name,
            "email" => $email,
            "phone" => $phone,
            "updated_at" => now()->utc()
        ];

        if (!empty($password))
        {
            $obj["password"] = password_hash($password, PASSWORD_DEFAULT);
        }

        DB::table($this->table)
            ->where("id", "=", $client_id)
            ->update($obj);

        return response()->json([
            "status" => "success",
            "message" => "Data has been updated.",
            "client" => (object) $obj
        ]);
    }

    public function edit()
    {
        $id = request()->id ?? 0;

        $client = DB::table($this->table)
            ->where("id", "=", $id)
            ->whereNull("deleted_at")
            ->first();

        if ($client == null)
        {
            abort(404);
        }

        return view("admin/clients/edit", [
            "client" => $client
        ]);
    }

    public function fetch()
    {
        $this->admin_auth();
        $search = request()->search ?? "";

        $time_zone = $_POST["time_zone"] ?? "";
        if (!empty($time_zone))
        {
            date_default_timezone_set($time_zone);
        }

        $clients = DB::table($this->table);
        if (!empty($search))
        {
            $clients = $clients->where(function ($query) use ($search) {
                $query->where("name", "LIKE", "%" . $search . "%")
                    ->orWhere("email", "LIKE", "%" . $search . "%")
                    ->orWhere("phone", "LIKE", "%" . $search . "%");
            });
        }

        $clients = $clients->whereNull("deleted_at")
            ->orderBy("id", "desc")
            ->paginate(50);

        $clients_arr = [];
        foreach ($clients as $client)
        {
            $obj = [
                "id" => $client->id ?? 0,
                "name" => $client->name ?? "",
                "email" => $client->email ?? "",
                "phone" => $client->phone ?? "",
                "created_at" => date("d M, Y h:i:s a", strtotime($client->created_at . " UTC")),
                "updated_at" => date("d M, Y h:i:s a", strtotime($client->updated_at . " UTC"))
            ];

            array_push($clients_arr, (object) $obj);
        }

        return response()->json([
            "status" => "success",
            "message" => "Data has been fetched.",
            "clients" => $clients_arr,
            "pages" => $clients->lastPage()
            // "pagination" => $clients->links()->render(),
        ]);
    }

    public function store()
    {
        $validator = Validator::make(request()->all(), [
            "name" => "required",
            "password" => "required"
        ]);

        if (!$validator->passes() && count($validator->errors()->all()) > 0)
        {
            return response()->json([
                "status" => "error",
                "message" => $validator->errors()->all()[0]
            ]);
        }

        $this->admin_auth();
        $name = request()->name ?? "";
        $email = request()->email ?? "";
        $phone = request()->phone ?? "";
        $password = request()->password ?? "";

        $exists_by = "";
        $client = DB::table($this->table);
        if (!empty($email))
        {
            $client = $client->where("email", "=", $email);
            $exists_by = "email";
        }
        if (!empty($phone))
        {
            $client = $client->where("phone", "=", $phone);
            $exists_by = "phone";
        }
        $client = $client->whereNull("deleted_at")->first();

        if ($client != null)
        {
            return response()->json([
                "status" => "error",
                "message" => "Client with same " . $exists_by . " already exists."
            ]);
        }

        $obj = [
            "name" => $name,
            "email" => $email,
            "phone" => $phone,
            "password" => password_hash($password, PASSWORD_DEFAULT),
            "created_at" => now()->utc(),
            "updated_at" => now()->utc()
        ];

        $obj["id"] = DB::table($this->table)
            ->insertGetId($obj);

        return response()->json([
            "status" => "success",
            "message" => "Data has been added.",
            "client" => (object) $obj
        ]);
    }

    public function add()
    {
        return view("admin/clients/add");
    }

    public function admin_index()
    {
        return view("admin/clients/index");
    }
}
