<?php

namespace App\Http\Controllers;

//Import PHPMailer classes into the global namespace
//These must be at the top of your script, not inside a function
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\SMTP;
use PHPMailer\PHPMailer\Exception;

// Include the JWT library
use Firebase\JWT\JWT;
use Firebase\JWT\Key;

use DB;

abstract class Controller
{
    protected $token_secret = "project-manager-token-secret";
    protected $admin_token_secret = "project-manager-admin-token-secret";
    // Secret key for signing the JWT
    private $client_jwt_secret = 'client-jwt-secret';

    protected function capitalize($str)
    {
        // split words by space
        $parts = explode(" ", $str);
         
        // loop through each word
        foreach ($parts as $key => $value)
        {
            // make first letter uppercase
            // and update the word
            $parts[$key] = ucfirst($value);
        }
         
        // join all words by space
        $str = implode(" ", $parts);

        return $str;
    }

    // Function to generate a JWT token
    protected function generate_jwt($id)
    {
        $issuedAt = time();
        $expirationTime = $issuedAt + (3600 * 24 * 30);  // jwt valid for 1 hour * 24 = 24 hours * 30 = 30 days

        $payload = [
            'iat' => $issuedAt,
            'exp' => $expirationTime,
            'id' => $id
        ];

        // Add "HS256" as the third parameter for encoding
        return JWT::encode($payload, $this->client_jwt_secret, 'HS256');
    }

    // Function to decode a JWT token
    private function decode_jwt($jwt)
    {
        try {
            $decoded = JWT::decode($jwt, new Key($this->client_jwt_secret, 'HS256'));
            return (array) $decoded;
        } catch (Exception $e) {
            return null;
        }
    }

    protected function auth_client()
    {
        $headers = apache_request_headers();
        $authHeader = isset($headers['Authorization']) ? $headers['Authorization'] : '';

        if ($authHeader) {
            list(, $token) = explode(' ', $authHeader);
            $decoded = $this->decode_jwt($token);

            if ($decoded && $decoded['id']) {
                // Token is valid, allow access

                $client = DB::table("clients")
                    ->where("id", "=", $decoded['id'])
                    ->whereNull("deleted_at")
                    ->first();

                if ($client == null)
                {
                    return response()->json([
                        "status" => "error",
                        "message" => "Un-authenticated."
                    ])->throwResponse();
                }

                return $client;
            } else {
                return response()->json([
                    "status" => "error",
                    "message" => "Un-authenticated."
                ])->throwResponse();
            }
        } else {
            return response()->json([
                "status" => "error",
                "message" => "Authorization header missing."
            ])->throwResponse();
        }
    }

    protected function admin_auth()
    {
        if (!auth()->check())
        {
            return response()->json([
                "status" => "error",
                "message" => "Un-authorized."
            ])->throwResponse();
        }

        if (auth()->user()->type != "super_admin")
        {
            return response()->json([
                "status" => "error",
                "message" => "Un-authorized."
            ])->throwResponse();
        }
    }

    protected function send_mail($to, $to_name, $subject, $body)
    {
        //Create an instance; passing `true` enables exceptions
        $mail = new PHPMailer(true);

        $settings = DB::table("settings")->get();
        if (count($settings) <= 0)
        {
            return "SMTP configurations not set.";
        }

        $settings_obj = new \stdClass();
        foreach ($settings as $setting)
        {
            $settings_obj->{$setting->key} = $setting->value;
        }

        try
        {
            //Server settings
            $mail->SMTPDebug = 0; // SMTP::DEBUG_SERVER;                      //Enable verbose debug output
            $mail->isSMTP();                                            //Send using SMTP
            $mail->Host       = $settings_obj->smtp_host;                     //Set the SMTP server to send through
            $mail->SMTPAuth   = true;                                   //Enable SMTP authentication
            $mail->Username   = $settings_obj->smtp_username;                     //SMTP username
            $mail->Password   = $settings_obj->smtp_password;                               //SMTP password
            $mail->SMTPSecure = PHPMailer::ENCRYPTION_SMTPS;            //Enable implicit TLS encryption
            $mail->Port       = $settings_obj->smtp_port;                                    //TCP port to connect to; use 587 if you have set `SMTPSecure = PHPMailer::ENCRYPTION_STARTTLS`

            //Recipients
            $mail->setFrom($settings_obj->smtp_from, $settings_obj->smtp_from_name);
            $mail->addAddress($to, $to_name);     //Add a recipient

            //Content
            $mail->isHTML(true);                                  //Set email format to HTML
            $mail->Subject = $subject;
            $mail->Body    = $body;
            $mail->AltBody = $body;

            $mail->send();
            return "";
            // echo 'Message has been sent';
        }
        catch (Exception $e)
        {
            return $mail->ErrorInfo;
            // echo "Message could not be sent. Mailer Error: {$mail->ErrorInfo}";
        }
    }
}
