<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;

use DB;
use Storage;
use Validator;

class TaskController extends Controller
{
    private $table = "tasks";

    public function destroy()
    {
        $validator = Validator::make(request()->all(), [
            "id" => "required"
        ]);

        if (!$validator->passes() && count($validator->errors()->all()) > 0)
        {
            return response()->json([
                "status" => "error",
                "message" => $validator->errors()->all()[0]
            ]);
        }

        $this->admin_auth();
        $id = request()->id ?? 0;

        $task = DB::table($this->table)
            ->where("id", "=", $id)
            ->whereNull("deleted_at")
            ->first();

        if ($task == null)
        {
            return response()->json([
                "status" => "error",
                "message" => "Task not found."
            ]);
        }

        DB::table($this->table)
            ->where("id", "=", $task->id)
            ->update([
                "deleted_at" => now()->utc()
            ]);

        return response()->json([
            "status" => "success",
            "message" => "Task has been deleted."
        ]);
    }

    public function update()
    {
        $validator = Validator::make(request()->all(), [
            "id" => "required",
            "title" => "required",
            "price" => "required",
            "status" => "required",
            "payment_status" => "required",
            "client_id" => "required",
            "project_id" => "required"
        ]);

        if (!$validator->passes() && count($validator->errors()->all()) > 0)
        {
            return response()->json([
                "status" => "error",
                "message" => $validator->errors()->all()[0]
            ]);
        }

        $this->admin_auth();
        $id = request()->id ?? 0;
        $title = request()->title ?? "";
        $description = request()->description ?? "";
        $price = request()->price ?? 0;
        $status = request()->status ?? "";
        $payment_status = request()->payment_status ?? "";
        $client_id = request()->client_id ?? 0;
        $project_id = request()->project_id ?? 0;

        if (!in_array($status, ["todo", "progress", "done"]))
        {
            return response()->json([
                "status" => "error",
                "message" => "In-valid 'status' value."
            ]);
        }

        if (!in_array($payment_status, ["paid", "not_paid", "discussed", "price_not_finalized"]))
        {
            return response()->json([
                "status" => "error",
                "message" => "In-valid 'payment_status' value."
            ]);
        }

        $task = DB::table($this->table)
            ->where("id", "=", $id)
            ->whereNull("deleted_at")
            ->first();

        if ($task == null)
        {
            return response()->json([
                "status" => "error",
                "message" => "Task not found."
            ]);
        }

        $task_id = $task->id;

        $client = DB::table("clients")
            ->where("id", "=", $client_id)
            ->whereNull("deleted_at")
            ->first();

        if ($client == null)
        {
            return response()->json([
                "status" => "error",
                "message" => "Client not found."
            ]);
        }

        $project = DB::table("projects")
            ->where("id", "=", $project_id)
            ->whereNull("deleted_at")
            ->first();

        if ($project == null)
        {
            return response()->json([
                "status" => "error",
                "message" => "Project not found."
            ]);
        }

        $exists_by = "";
        $task = DB::table($this->table);
        if (!empty($title))
        {
            $task = $task->where("title", "=", $title)
                ->where("id", "!=", $task_id);
            $exists_by = "title";
        }
        $task = $task->whereNull("deleted_at")->first();

        if ($task != null)
        {
            return response()->json([
                "status" => "error",
                "message" => "Task with same " . $exists_by . " already exists."
            ]);
        }

        $obj = [
            "title" => $title,
            "description" => $description,
            "price" => $price,
            "status" => $status,
            "payment_status" => $payment_status,
            "client_id" => $client->id,
            "project_id" => $project->id,
            "updated_at" => now()->utc()
        ];

        DB::table($this->table)
            ->where("id", "=", $task_id)
            ->update($obj);

        return response()->json([
            "status" => "success",
            "message" => "Data has been updated.",
            "task" => (object) $obj
        ]);
    }

    public function edit()
    {
        $id = request()->id ?? 0;

        $task = DB::table($this->table)
            ->where("id", "=", $id)
            ->whereNull("deleted_at")
            ->first();

        if ($task == null)
        {
            abort(404);
        }

        $clients = DB::table("clients")
            ->whereNull("deleted_at")
            ->orderBy("name", "asc")
            ->get();

        $projects = DB::table("projects")
            ->whereNull("deleted_at")
            ->orderBy("name", "asc")
            ->get();

        return view("admin/tasks/edit", [
            "task" => $task,
            "clients" => $clients,
            "projects" => $projects
        ]);
    }

    public function fetch_admin()
    {
        $this->admin_auth();
        $search = request()->search ?? "";

        $time_zone = $_POST["time_zone"] ?? "";
        if (!empty($time_zone))
        {
            date_default_timezone_set($time_zone);
        }

        $tasks = DB::table($this->table)
            ->select($this->table . ".*", "clients.name AS client_name",
                "clients.email AS client_email", "clients.phone AS client_phone",
                "projects.name AS project_name")
            ->join("clients", "clients.id", "=", $this->table . ".client_id")
            ->join("projects", "projects.id", "=", $this->table . ".project_id");
        if (!empty($search))
        {
            $table = $this->table;
            $tasks = $tasks->where(function ($query) use ($search, $table) {
                $query->where($table . ".title", "LIKE", "%" . $search . "%")
                    ->orWhere($table . ".description", "LIKE", "%" . $search . "%");
            });
        }

        $tasks = $tasks->whereNull($this->table . ".deleted_at")
            ->orderBy($this->table . ".id", "desc")
            ->paginate(50);

        $tasks_arr = [];
        foreach ($tasks as $task)
        {
            $obj = [
                "id" => $task->id ?? 0,
                "title" => $task->title ?? "",
                "description" => $task->description ?? "",
                "price" => $task->price ?? 0,
                "status" => $task->status ?? "",
                "payment_status" => $task->payment_status ?? "",
                "client" => (object) [
                    "id" => $task->client_id ?? 0,
                    "name" => $task->client_name ?? "",
                    "email" => $task->client_email ?? "",
                    "phone" => $task->client_phone ?? ""
                ],
                "project" => (object) [
                    "id" => $task->project_id ?? 0,
                    "name" => $task->project_name ?? ""
                ],
                "created_at" => date("d M, Y h:i:s a", strtotime($task->created_at . " UTC")),
                "updated_at" => date("d M, Y h:i:s a", strtotime($task->updated_at . " UTC"))
            ];

            $obj["payment_status_str"] = $this->capitalize(str_replace("_", " ", $obj["payment_status"]));

            array_push($tasks_arr, (object) $obj);
        }

        return response()->json([
            "status" => "success",
            "message" => "Data has been fetched.",
            "tasks" => $tasks_arr,
            "pages" => $tasks->lastPage()
        ]);
    }

    public function store()
    {
        $validator = Validator::make(request()->all(), [
            "title" => "required",
            "price" => "required",
            "status" => "required",
            "payment_status" => "required",
            "client_id" => "required",
            "project_id" => "required"
        ]);

        if (!$validator->passes() && count($validator->errors()->all()) > 0)
        {
            return response()->json([
                "status" => "error",
                "message" => $validator->errors()->all()[0]
            ]);
        }

        $this->admin_auth();
        $title = request()->title ?? "";
        $description = request()->description ?? "";
        $price = request()->price ?? 0;
        $status = request()->status ?? "";
        $payment_status = request()->payment_status ?? "";
        $client_id = request()->client_id ?? 0;
        $project_id = request()->project_id ?? 0;

        if (!in_array($status, ["todo", "progress", "done"]))
        {
            return response()->json([
                "status" => "error",
                "message" => "In-valid 'status' value."
            ]);
        }

        if (!in_array($payment_status, ["paid", "not_paid", "price_not_finalized"]))
        {
            return response()->json([
                "status" => "error",
                "message" => "In-valid 'payment_status' value."
            ]);
        }

        $client = DB::table("clients")
            ->where("id", "=", $client_id)
            ->whereNull("deleted_at")
            ->first();

        if ($client == null)
        {
            return response()->json([
                "status" => "error",
                "message" => "Client not found."
            ]);
        }

        $project = DB::table("projects")
            ->where("id", "=", $project_id)
            ->whereNull("deleted_at")
            ->first();

        if ($project == null)
        {
            return response()->json([
                "status" => "error",
                "message" => "Project not found."
            ]);
        }

        $exists_by = "";
        $task = DB::table($this->table);
        if (!empty($title))
        {
            $task = $task->where("title", "=", $title);
            $exists_by = "title";
        }
        $task = $task->whereNull("deleted_at")->first();

        if ($task != null)
        {
            return response()->json([
                "status" => "error",
                "message" => "Task with same " . $exists_by . " already exists."
            ]);
        }

        $obj = [
            "title" => $title,
            "description" => $description,
            "price" => $price,
            "status" => $status,
            "payment_status" => $payment_status,
            "client_id" => $client->id,
            "project_id" => $project->id,
            "created_at" => now()->utc(),
            "updated_at" => now()->utc()
        ];

        $obj["id"] = DB::table($this->table)
            ->insertGetId($obj);

        return response()->json([
            "status" => "success",
            "message" => "Data has been added.",
            "task" => (object) $obj
        ]);
    }

    public function add()
    {
        $clients = DB::table("clients")
            ->whereNull("deleted_at")
            ->orderBy("name", "asc")
            ->get();

        $projects = DB::table("projects")
            ->whereNull("deleted_at")
            ->orderBy("name", "asc")
            ->get();

        return view("admin/tasks/add", [
            "clients" => $clients,
            "projects" => $projects
        ]);
    }

    public function admin_index()
    {
        return view("admin/tasks/index");
    }
}
