function Home() {

    const [projects, setProjects] = React.useState([])
    const [pages, setPages] = React.useState(0)
    const [timer, setTimer] = React.useState(null)
    const [search, setSearch] = React.useState("")
    const [loading, setLoading] = React.useState(false)

    async function onInit() {
        try {
            setLoading(true)

            const urlSearchParams = new URLSearchParams(window.location.search)
            const page = urlSearchParams.get("page") || 1

            const formData = new FormData()
            formData.append("page", page)
            formData.append("search", search)

            const response = await axios.post(
                baseUrl + "/api/projects/fetch",
                formData,
                {
                    headers: {
                        Authorization: "Bearer " + localStorage.getItem(accessTokenKey)
                    }
                }
            )

            if (response.data.status == "success") {
                setProjects(response.data.projects)
                setPages(response.data.pages)
            } else {
                swal.fire("Error", response.data.message, "error")
            }
        } catch (exp) {
            swal.fire("Error", exp.message, "error")
        } finally {
            setLoading(false)
        }
    }

    // function to be called whenever input field text value is changed
    function doSearch() {
        // get the input field DOM in a separate variable
        const self = event.target

        // if user types another key before 500 milliseconds,
        // then remove the timer, so the function will not gets called
        clearTimeout(timer)

        // this will create a timer every time input field has a value
        setTimer(setTimeout(function () {
            // write your own logic when user stops typing
            setSearch(self.value)
        }, 500))
    }

    React.useEffect(function () {
        onInit()
    }, [search])

    return (
        <div className="container mt-5 mb-5">
            <div className="row mb-5">
                <div className="col-12 text-center">
                    <h1>Your Projects</h1>
                </div>
            </div>

            <div className="row mb-3">
                <div className="col-12">
                    <input type="text" name="search" className="form-control" placeholder="Search projects here..."
                        onInput={ doSearch }
                        autoComplete="off" />
                </div>
            </div>

            <div className="row">
                <div className="col-12">
                    <table className="table table-bordered table-responsive">
                        <thead>
                            <tr>
                                <th>Project ID</th>
                                <th>Project Name</th>
                                <th>Last Updated</th>
                                <th>Actions</th>
                            </tr>
                        </thead>

                        <tbody>
                            { loading && (
                                <tr>
                                    <td colSpan="4">Loading...</td>
                                </tr>
                            ) }
                            
                            { !loading && projects.length <= 0 && (
                                <tr>
                                    <td colSpan="4">No project found.</td>
                                </tr>
                            ) }
                            
                            { projects.map(function (project) {
                                return (
                                    <tr key={ `project-${ project.id }` }>
                                        <td>{ project.id }</td>
                                        <td>{ project.name }</td>
                                        <td>{ project.updated_at }</td>
                                        <td>
                                            <a href={ `${ baseUrl }/projects/${ project.id }` }>View tasks</a>
                                        </td>
                                    </tr>
                                )
                            }) }
                        </tbody>
                    </table>
                </div>
            </div>

            <div className="row mt-3">
                <div className="col-12" id="pagination">
                    <nav>
                        <ul className="pagination">
                            { Pagination(pages, '/') }
                        </ul>
                    </nav>
                </div>
            </div>
        </div>
    )
}

ReactDOM.createRoot(
    document.getElementById("home-app")
).render(<Home />)