function Project() {

    const [project, setProject] = React.useState(null)
    const [tasks, setTasks] = React.useState([])
    const [selectedTask, setSelectedTask] = React.useState(null)
    const [pages, setPages] = React.useState(0)
    const [timer, setTimer] = React.useState(null)
    const [search, setSearch] = React.useState("")
    const [loading, setLoading] = React.useState(false)

    async function onInit() {
        try {
            setLoading(true)

            const urlSearchParams = new URLSearchParams(window.location.search)
            const page = urlSearchParams.get("page") || 1

            const formData = new FormData()
            formData.append("id", id)
            formData.append("page", page)
            formData.append("search", search)

            const response = await axios.post(
                baseUrl + "/api/projects/detail",
                formData,
                {
                    headers: {
                        Authorization: "Bearer " + localStorage.getItem(accessTokenKey)
                    }
                }
            )

            if (response.data.status == "success") {
                setProject(response.data.project)
                setTasks(response.data.tasks)
                setPages(response.data.pages)
            } else {
                swal.fire("Error", response.data.message, "error")
            }
        } catch (exp) {
            swal.fire("Error", exp.message, "error")
        } finally {
            setLoading(false)
        }
    }

    // function to be called whenever input field text value is changed
    function doSearch() {
        // get the input field DOM in a separate variable
        const self = event.target

        // if user types another key before 500 milliseconds,
        // then remove the timer, so the function will not gets called
        clearTimeout(timer)

        // this will create a timer every time input field has a value
        setTimer(setTimeout(function () {
            // write your own logic when user stops typing
            setSearch(self.value)
        }, 500))
    }

    React.useEffect(function () {
        onInit()
    }, [search])

    return (
        <>
            <div className="container mt-5 mb-5">

                <div className="row mb-5">
                    <div className="col-12 text-center">
                        <h1>{ project?.name || "" }</h1>
                    </div>
                </div>

                <div className="row mb-3">
                    <div className="col-12">
                        <input type="text" name="search" className="form-control" placeholder="Search tasks here..."
                            onInput={ doSearch }
                            autoComplete="off" />
                    </div>
                </div>

                <div className="row">
                    <div className="col-12">
                        <table className="table table-bordered table-responsive">
                            <thead>
                                <tr>
                                    <th>Task</th>
                                    <th>Price</th>
                                    <th>Status</th>
                                    <th>Payment Status</th>
                                    <th>Last Updated</th>
                                    <th>Actions</th>
                                </tr>
                            </thead>

                            <tbody>
                                { loading && (
                                    <tr>
                                        <td colSpan="4">Loading...</td>
                                    </tr>
                                ) }
                                
                                { !loading && tasks.length <= 0 && (
                                    <tr>
                                        <td colSpan="4">No task found.</td>
                                    </tr>
                                ) }

                                { tasks.map(function (task) {
                                    return (
                                        <tr key={ `task-${ task.id }` }>
                                            <td>{ task.title }</td>
                                            <td>${ task.price }</td>
                                            <td className="capitalize">{ task.status }</td>
                                            <td>{ task.payment_status_str }</td>
                                            <td>{ task.updated_at }</td>
                                            <td>
                                                <a href="#" onClick={ function () {
                                                    event.preventDefault()
                                                    setSelectedTask(task)
                                                    $("#task-detail-modal").modal("show")
                                                } }>Detail</a>
                                            </td>
                                        </tr>
                                    )
                                }) }
                            </tbody>
                        </table>
                    </div>
                </div>

                <div className="row mt-3">
                    <div className="col-12" id="pagination">
                        <nav>
                            <ul className="pagination">
                                { Pagination(pages, 'projects/' + project?.id) }
                            </ul>
                        </nav>
                    </div>
                </div>
            </div>

            <div className="modal fade" id="task-detail-modal">
                <div className="modal-dialog modal-lg">
                    <div className="modal-content">
                        <div className="modal-header">
                            <h5 className="modal-title">{ selectedTask?.title || "" }</h5>
                            <button type="button" className="btn-close" data-bs-dismiss="modal"></button>
                        </div>

                        <div className="modal-body">
                            <div>{ HTMLReactParser(selectedTask?.description || "") }</div>
                        </div>

                        <div className="modal-footer">
                            <button type="button" className="btn btn-primary" data-bs-dismiss="modal">Okay</button>
                        </div>
                    </div>
                </div>
            </div>
        </>
    )
}

ReactDOM.createRoot(
    document.getElementById("project-app")
).render(<Project />)