@extends ("admin/layouts/app")
@section ("title", "Edit project")

@section ("main")

  <div class="pagetitle">
    <h1>Edit project</h1>
    <nav>
      <ol class="breadcrumb">
        <li class="breadcrumb-item">Projects</li>
        <li class="breadcrumb-item">Edit</li>
        <li class="breadcrumb-item active">{{ $project->id }}</li>
      </ol>
    </nav>
  </div>

  <section class="section">
    <div class="row">
      <div class="col-12">
        <div class="card">
          <div class="card-body">
            <h5 class="card-title">Project information</h5>

            <form onsubmit="editProject()" id="form-edit-project">
                <input type="hidden" name="id" value="{{ $project->id }}" required />

              <div class="row mb-3">
                <label class="col-sm-2 col-form-label">Name</label>
                <div class="col-sm-10">
                  <input type="text" class="form-control" name="name" value="{{ $project->name }}" required />
                </div>
              </div>

              <div class="row mb-3">
                <label class="col-sm-2 col-form-label">Client</label>
                <div class="col-sm-10">
                    <select name="client_id" class="form-control" required>
                        <option value="0">Select client</option>

                        @foreach ($clients as $client)
                            <option value="{{ $client->id }}" {{ $project->client_id == $client->id ? "selected" : "" }}>
                                @if ($client->name)
                                    {{ $client->name }} |
                                @endif

                                @if ($client->email)
                                    {{ $client->email }} |
                                @endif

                                @if ($client->phone)
                                    {{ $client->phone }}
                                @endif
                            </option>
                        @endforeach
                    </select>
                </div>
              </div>

              <input type="submit" name="submit" class="btn btn-outline-warning" value="Update" />
            </form>
          </div>
        </div>
      </div>
    </div>
  </section>

  <script>

    async function editProject() {
      event.preventDefault()

      const form = event.target
      const formData = new FormData(form)
      form.submit.setAttribute("disabled", "disabled")

      try {
        const response = await axios.post(
          baseUrl + "/api/admin/projects/update",
          formData,
          {
            headers: {
              Authorization: "Bearer " + localStorage.getItem(accessTokenKey)
            }
          }
        )

        if (response.data.status == "success") {
          swal.fire("Update project", response.data.message, "success")
        } else {
          swal.fire("Error", response.data.message, "error")
        }
      } catch (exp) {
        swal.fire("Error", exp.message, "error")
      } finally {
        form.submit.removeAttribute("disabled")
      }
    }
  </script>

@endsection