@extends ("admin/layouts/app")
@section ("title", "Projects")

@section ("main")

  <div class="pagetitle">
    <div style="display: flex;">
      <h1>Projects</h1>
      <a href="{{ url('/admin/projects/add') }}" class="btn btn-outline-primary btn-sm"
        style="margin-left: 15px;">Add project</a>
    </div>

    <nav>
      <ol class="breadcrumb">
        <li class="breadcrumb-item active">Projects</li>
      </ol>
    </nav>
  </div>

  <section class="section" id="projects-app">

  </section>

  <script type="text/babel">
    function Projects() {
      const styles = {
        editBtn: {
          marginBottom: "10px"
        }
      }

      const [projects, setProjects] = React.useState([])
      const [pages, setPages] = React.useState(0)
      const [timer, setTimer] = React.useState(null)
      const [search, setSearch] = React.useState("")

      async function onInit() {
        try {
          const urlSearchParams = new URLSearchParams(window.location.search)
          const page = urlSearchParams.get("page") || 1

          const timeZone = Intl.DateTimeFormat().resolvedOptions().timeZone

          const formData = new FormData()
          formData.append("time_zone", timeZone)
          formData.append("search", search)
          formData.append("page", page)

          const response = await axios.post(
            baseUrl + "/api/admin/projects/fetch",
            formData,
            {
              headers: {
                Authorization: "Bearer " + localStorage.getItem(accessTokenKey)
              }
            }
          )

          if (response.data.status == "success") {
            setProjects(response.data.projects)
            setPages(response.data.pages)
          } else {
            swal.fire("Error", response.data.message, "error")
          }
        } catch (exp) {
          swal.fire("Error", exp.message, "error")
        }
      }

      React.useEffect(function () {
        onInit()
      }, [search])

      function deleteProject(id, name) {
        swal.fire({
          title: "Delete project: " + name,
          text: "Are you sure you want to delete this project ?",
          icon: "warning",
          showCancelButton: true,
          confirmButtonColor: "#3085d6",
          cancelButtonColor: "#d33",
          confirmButtonText: "Yes, delete it!"
        }).then(async function (result) {
          if (result.isConfirmed) {
            try {
              const formData = new FormData()
              formData.append("id", id)

              const response = await axios.post(
                baseUrl + "/api/admin/projects/delete",
                formData,
                {
                  headers: {
                    Authorization: "Bearer " + localStorage.getItem(accessTokenKey)
                  }
                }
              )

              if (response.data.status == "success") {
                const tempProjects = [...projects]
                for (let a = 0; a < tempProjects.length; a++) {
                  if (tempProjects[a].id == id) {
                    tempProjects.splice(a, 1)
                  }
                }
                setProjects(tempProjects)
              } else {
                swal.fire("Error", response.data.message, "error")
              }
            } catch (exp) {
              swal.fire("Error", exp.message, "error")
            }
          }
        })
      }

      // function to be called whenever input field text value is changed
      function doSearch() {
        // get the input field DOM in a separate variable
        const self = event.target
         
        // if user types another key before 500 milliseconds,
        // then remove the timer, so the function will not gets called
        clearTimeout(timer)
     
        // this will create a timer every time input field has a value
        setTimer(setTimeout(function () {
          // write your own logic when user stops typing
          setSearch(self.value)
        }, 500))
      }

      return (
        <>
          <div className="row mb-3">
            <div className="col-12">
              <input type="text" name="search" className="form-control" placeholder="Search projects here..."
                onInput={ doSearch }
                autoComplete="off" />
            </div>
          </div>

          <div className="row">
            <div className="col-12">
              <table className="table table-bordered table-responsive">
                <thead>
                  <tr>
                    <th>Name</th>
                    <th>Client</th>
                    <th>Created at</th>
                    <th>Updated at</th>
                    <th>Actions</th>
                  </tr>
                </thead>

                <tbody>
                  { projects.map(function (project) {
                    return (
                      <tr key={`project-${ project.id }`}>
                        <td>{ project.name }</td>
                        <td>
                          Name = { project.client.name } <br />
                          E-mail = { project.client.email } <br />
                          Phone = { project.client.phone }
                        </td>
                        <td>{ project.created_at }</td>
                        <td>{ project.updated_at }</td>
                        <td>
                          <a href={`${ baseUrl }/admin/projects/edit/${ project.id }`} style={ styles.editBtn } className="btn btn-outline-warning">Edit</a>&nbsp;
                          
                          <button type="button" className="btn btn-outline-danger" onClick={ function () {
                            deleteProject(project.id, project.name)
                          } }>Delete</button>
                        </td>
                      </tr>
                    )
                  }) }
                </tbody>
              </table>
            </div>
          </div>

          <div className="row mt-3">
            <div className="col-12" id="pagination">
              <nav>
                <ul className="pagination">
                  { Pagination(pages, 'admin/projects') }
                </ul>
              </nav>
            </div>
          </div>
        </>
      )
    }

    ReactDOM.createRoot(
      document.getElementById("projects-app")
    ).render(<Projects />)
  </script>

@endsection