@extends ("admin/layouts/app")
@section ("title", "Add task")

@section ("main")

    <input type="hidden" id="clients" value="{{ json_encode($clients) }}" />
    <input type="hidden" id="projects" value="{{ json_encode($projects) }}" />

  <div class="pagetitle">
    <h1>Add task</h1>
    <nav>
      <ol class="breadcrumb">
        <li class="breadcrumb-item">Tasks</li>
        <li class="breadcrumb-item active">Add</li>
      </ol>
    </nav>
  </div>

  <section class="section">
    <div class="row">
      <div class="col-12">
        <div class="card">
          <div class="card-body">
            <h5 class="card-title">Task information</h5>

            <form onsubmit="addTask()" id="form-add-task">
              <div class="row mb-3">
                <label class="col-sm-2 col-form-label">Title</label>
                <div class="col-sm-10">
                  <input type="text" class="form-control" name="title" required />
                </div>
              </div>

                <div class="row mb-3">
                    <label class="col-sm-2 col-form-label">Description</label>
                    <div class="col-sm-10">
                        <textarea name="description" class="form-control"></textarea>
                    </div>
                </div>

                <div class="row mb-3">
                    <label class="col-sm-2 col-form-label">Price ($)</label>
                    <div class="col-sm-10">
                        <input type="number" name="price" class="form-control" required />
                    </div>
                </div>

                <div class="row mb-3">
                    <label class="col-sm-2 col-form-label">Status</label>
                    <div class="col-sm-10">
                        <select name="status" class="form-control" required>
                            <option value="todo">Todo</option>
                            <option value="progress">Progress</option>
                            <option value="done">Done</option>
                        </select>
                    </div>
                </div>

                <div class="row mb-3">
                    <label class="col-sm-2 col-form-label">Payment Status</label>
                    <div class="col-sm-10">
                        <select name="payment_status" class="form-control" required>
                            <option value="paid">Paid</option>
                            <option value="not_paid">Not Paid</option>
                            <option value="price_not_finalized">Price not finalized</option>
                        </select>
                    </div>
                </div>

              <div class="row mb-3">
                <label class="col-sm-2 col-form-label">Client</label>
                <div class="col-sm-10">
                    <select name="client_id" class="form-control" required onchange="onClientSelected()">
                        <option value="0">Select client</option>

                        @foreach ($clients as $client)
                            <option value="{{ $client->id }}">
                                @if ($client->name)
                                    {{ $client->name }} |
                                @endif

                                @if ($client->email)
                                    {{ $client->email }} |
                                @endif

                                @if ($client->phone)
                                    {{ $client->phone }}
                                @endif
                            </option>
                        @endforeach
                    </select>
                </div>
              </div>

                <div class="row mb-3">
                    <label class="col-sm-2 col-form-label">Project</label>
                    <div class="col-sm-10">
                        <select name="project_id" class="form-control" required>
                            <option value="0">Select project</option>
                        </select>
                    </div>
                </div>

              <input type="submit" name="submit" class="btn btn-outline-primary" value="Add" />
            </form>
          </div>
        </div>
      </div>
    </div>
  </section>

  <script>

    let clients = document.getElementById("clients").value
    clients = JSON.parse(clients)

    let projects = document.getElementById("projects").value
    projects = JSON.parse(projects)

    function onClientSelected() {
        const clientId = event.target.value

        let html = ""
        html += `<option value="0">Select project</option>`
        for (let a = 0; a < projects.length; a++) {
            if (projects[a].client_id == clientId) {
                html += `<option value="` + projects[a].id + `">` + projects[a].name + `</option>`
            }
        }
        document.querySelector("select[name='project_id']").innerHTML = html
    }

    async function addTask() {
      event.preventDefault()

      const form = event.target
      const formData = new FormData(form)
      form.submit.setAttribute("disabled", "disabled")

      try {
        const response = await axios.post(
          baseUrl + "/api/admin/tasks/add",
          formData,
          {
            headers: {
              Authorization: "Bearer " + localStorage.getItem(accessTokenKey)
            }
          }
        )

        if (response.data.status == "success") {
          swal.fire("Add task", response.data.message, "success")
        } else {
          swal.fire("Error", response.data.message, "error")
        }
      } catch (exp) {
        swal.fire("Error", exp.message, "error")
      } finally {
        form.submit.removeAttribute("disabled")
      }
    }

    $("textarea[name='description']").richText()
  </script>

@endsection