@extends ("admin/layouts/app")
@section ("title", "Edit task")

@section ("main")

    <input type="hidden" id="clients" value="{{ json_encode($clients) }}" />
    <input type="hidden" id="projects" value="{{ json_encode($projects) }}" />

  <div class="pagetitle">
    <h1>Edit task</h1>
    <nav>
      <ol class="breadcrumb">
        <li class="breadcrumb-item">Tasks</li>
        <li class="breadcrumb-item">Edit</li>
        <li class="breadcrumb-item active">{{ $task->id }}</li>
      </ol>
    </nav>
  </div>

  <section class="section">
    <div class="row">
      <div class="col-12">
        <div class="card">
          <div class="card-body">
            <h5 class="card-title">Task information</h5>

            <form onsubmit="updateTask()" id="form-edit-task">
                <input type="hidden" name="id" value="{{ $task->id }}" required />
              <div class="row mb-3">
                <label class="col-sm-2 col-form-label">Title</label>
                <div class="col-sm-10">
                  <input type="text" class="form-control" name="title" value="{{ $task->title }}" required />
                </div>
              </div>

                <div class="row mb-3">
                    <label class="col-sm-2 col-form-label">Description</label>
                    <div class="col-sm-10">
                        <textarea name="description" class="form-control">{{ $task->description }}</textarea>
                    </div>
                </div>

                <div class="row mb-3">
                    <label class="col-sm-2 col-form-label">Price ($)</label>
                    <div class="col-sm-10">
                        <input type="number" name="price" class="form-control" value="{{ $task->price }}" required />
                    </div>
                </div>

                <div class="row mb-3">
                    <label class="col-sm-2 col-form-label">Status</label>
                    <div class="col-sm-10">
                        <select name="status" class="form-control" required>
                            <option value="todo" {{ $task->status == "todo" ? "selected" : "" }}>Todo</option>
                            <option value="progress" {{ $task->status == "progress" ? "selected" : "" }}>Progress</option>
                            <option value="done" {{ $task->status == "done" ? "selected" : "" }}>Done</option>
                        </select>
                    </div>
                </div>

                <div class="row mb-3">
                    <label class="col-sm-2 col-form-label">Payment Status</label>
                    <div class="col-sm-10">
                        <select name="payment_status" class="form-control" required>
                            <option value="paid" {{ $task->payment_status == "paid" ? "selected" : "" }}>Paid</option>
                            <option value="not_paid" {{ $task->payment_status == "not_paid" ? "selected" : "" }}>Not Paid</option>
                            <option value="price_not_finalized" {{ $task->payment_status == "price_not_finalized" ? "selected" : "" }}>Price not finalized</option>
                        </select>
                    </div>
                </div>

              <div class="row mb-3">
                <label class="col-sm-2 col-form-label">Client</label>
                <div class="col-sm-10">
                    <select name="client_id" class="form-control" required onchange="onClientSelected()">
                        <option value="0">Select client</option>

                        @foreach ($clients as $client)
                            <option value="{{ $client->id }}" {{ $task->client_id == $client->id ? "selected" : "" }}>
                                @if ($client->name)
                                    {{ $client->name }} |
                                @endif

                                @if ($client->email)
                                    {{ $client->email }} |
                                @endif

                                @if ($client->phone)
                                    {{ $client->phone }}
                                @endif
                            </option>
                        @endforeach
                    </select>
                </div>
              </div>

                <div class="row mb-3">
                    <label class="col-sm-2 col-form-label">Project</label>
                    <div class="col-sm-10">
                        <select name="project_id" class="form-control" required>
                            <option value="0">Select project</option>

                            @foreach ($projects as $project)
                                <option value="{{ $project->id }}" {{ $task->project_id == $project->id ? "selected" : "" }}>
                                    {{ $project->name }}
                                </option>
                            @endforeach
                        </select>
                    </div>
                </div>

              <input type="submit" name="submit" class="btn btn-outline-warning" value="Update" />
            </form>
          </div>
        </div>
      </div>
    </div>
  </section>

  <script>

    let clients = document.getElementById("clients").value
    clients = JSON.parse(clients)

    let projects = document.getElementById("projects").value
    projects = JSON.parse(projects)

    function onClientSelected() {
        const clientId = event.target.value

        let html = ""
        html += `<option value="0">Select project</option>`
        for (let a = 0; a < projects.length; a++) {
            if (projects[a].client_id == clientId) {
                html += `<option value="` + projects[a].id + `">` + projects[a].name + `</option>`
            }
        }
        document.querySelector("select[name='project_id']").innerHTML = html
    }

    async function updateTask() {
      event.preventDefault()

      const form = event.target
      const formData = new FormData(form)
      form.submit.setAttribute("disabled", "disabled")

      try {
        const response = await axios.post(
          baseUrl + "/api/admin/tasks/update",
          formData,
          {
            headers: {
              Authorization: "Bearer " + localStorage.getItem(accessTokenKey)
            }
          }
        )

        if (response.data.status == "success") {
          swal.fire("Update task", response.data.message, "success")
        } else {
          swal.fire("Error", response.data.message, "error")
        }
      } catch (exp) {
        swal.fire("Error", exp.message, "error")
      } finally {
        form.submit.removeAttribute("disabled")
      }
    }

    $("textarea[name='description']").richText()
  </script>

@endsection