@extends ("admin/layouts/app")
@section ("title", "Tasks")

@section ("main")

  <div class="pagetitle">
    <div style="display: flex;">
      <h1>Tasks</h1>
      <a href="{{ url('/admin/tasks/add') }}" class="btn btn-outline-primary btn-sm"
        style="margin-left: 15px;">Add task</a>
    </div>

    <nav>
      <ol class="breadcrumb">
        <li class="breadcrumb-item active">Tasks</li>
      </ol>
    </nav>
  </div>

  <section class="section" id="tasks-app">

  </section>

  <script type="text/babel">
    function Tasks() {
      const styles = {
        editBtn: {
          marginBottom: "10px"
        }
      }

      const [tasks, setTasks] = React.useState([])
      const [pages, setPages] = React.useState(0)
      const [timer, setTimer] = React.useState(null)
      const [search, setSearch] = React.useState("")

      async function onInit() {
          try {
            const urlSearchParams = new URLSearchParams(window.location.search)
            const page = urlSearchParams.get("page") || 1

            const timeZone = Intl.DateTimeFormat().resolvedOptions().timeZone

            const formData = new FormData()
            formData.append("time_zone", timeZone)
            formData.append("search", search)
            formData.append("page", page)

            const response = await axios.post(
              baseUrl + "/api/admin/tasks/fetch",
              formData,
              {
                headers: {
                  Authorization: "Bearer " + localStorage.getItem(accessTokenKey)
                }
              }
            )

            if (response.data.status == "success") {
              setTasks(response.data.tasks)
              setPages(response.data.pages)
            } else {
              swal.fire("Error", response.data.message, "error")
            }
          } catch (exp) {
            swal.fire("Error", exp.message, "error")
          }
      }

      React.useEffect(function () {
          onInit()
      }, [search])

      function deleteTask(id, title) {
        swal.fire({
          title: "Delete task: " + title,
          text: "Are you sure you want to delete this task ?",
          icon: "warning",
          showCancelButton: true,
          confirmButtonColor: "#3085d6",
          cancelButtonColor: "#d33",
          confirmButtonText: "Yes, delete it!"
        }).then(async function (result) {
          if (result.isConfirmed) {
            try {
              const formData = new FormData()
              formData.append("id", id)

              const response = await axios.post(
                baseUrl + "/api/admin/tasks/delete",
                formData,
                {
                  headers: {
                    Authorization: "Bearer " + localStorage.getItem(accessTokenKey)
                  }
                }
              )

              if (response.data.status == "success") {
                const tempTasks = [...tasks]
                for (let a = 0; a < tempTasks.length; a++) {
                  if (tempTasks[a].id == id) {
                    tempTasks.splice(a, 1)
                  }
                }
                setTasks(tempTasks)
              } else {
                swal.fire("Error", response.data.message, "error")
              }
            } catch (exp) {
              swal.fire("Error", exp.message, "error")
            }
          }
        })
      }

      // function to be called whenever input field text value is changed
      function doSearch() {
        // get the input field DOM in a separate variable
        const self = event.target
         
        // if user types another key before 500 milliseconds,
        // then remove the timer, so the function will not gets called
        clearTimeout(timer)
     
        // this will create a timer every time input field has a value
        setTimer(setTimeout(function () {
          // write your own logic when user stops typing
          setSearch(self.value)
        }, 500))
      }

      return (
        <>
          <div className="row mb-3">
            <div className="col-12">
              <input type="text" name="search" className="form-control" placeholder="Search tasks here..."
                onInput={ doSearch }
                autoComplete="off" />
            </div>
          </div>

          <div className="row">
            <div className="col-12">
              <table className="table table-bordered table-responsive">
                <thead>
                  <tr>
                    <th>Title</th>
                    <th>Price ($)</th>
                    <th>Status</th>
                    <th>Payment status</th>
                    <th>Client</th>
                    <th>Project</th>
                    <th>Created at</th>
                    <th>Updated at</th>
                    <th>Actions</th>
                  </tr>
                </thead>

                <tbody>
                  { tasks.map(function (task) {
                    return (
                      <tr key={`task-${ task.id }`}>
                        <td>{ task.title }</td>
                        <td>{ task.price }</td>
                        <td className="capitalize">{ task.status }</td>
                        <td>{ task.payment_status_str }</td>
                        <td>
                          Name = { task.client.name } <br />
                          E-mail = { task.client.email } <br />
                          Phone = { task.client.phone }
                        </td>
                        <td>{ task.project.name }</td>
                        <td>{ task.created_at }</td>
                        <td>{ task.updated_at }</td>
                        <td>
                          <a href={`${ baseUrl }/admin/tasks/edit/${ task.id }`} style={ styles.editBtn } className="btn btn-outline-warning">Edit</a>&nbsp;
                          
                          <button type="button" className="btn btn-outline-danger" onClick={ function () {
                            deleteTask(task.id, task.title)
                          } }>Delete</button>
                        </td>
                      </tr>
                    )
                  }) }
                </tbody>
              </table>
            </div>
          </div>

          <div className="row mt-3">
            <div className="col-12" id="pagination">
              <nav>
                <ul className="pagination">
                  { Pagination(pages, 'admin/tasks') }
                </ul>
              </nav>
            </div>
          </div>
        </>
      )
    }

    ReactDOM.createRoot(
      document.getElementById("tasks-app")
    ).render(<Tasks />)
  </script>

@endsection