<?php

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;

use App\Http\Controllers\AdminController;
use App\Http\Controllers\UserController;
use App\Http\Controllers\MessagesController;
use App\Http\Controllers\ClientController;
use App\Http\Controllers\ProjectController;
use App\Http\Controllers\TaskController;

// Route::get('/user', function (Request $request) {
//     return $request->user();
// })->middleware('auth:sanctum');

Route::post("/projects/detail", [ProjectController::class, "detail"]);
Route::post("/projects/fetch", [ProjectController::class, "fetch"]);

Route::post("/verify-email", [UserController::class, "verify_email"]);
Route::post("/reset-password", [UserController::class, "reset_password"]);
Route::post("/send-password-reset-link", [UserController::class, "send_password_reset_link"]);
Route::post("/login", [ClientController::class, "login"]);
Route::post("/register", [UserController::class, "register"]);
Route::post("/admin/login", [AdminController::class, "login"]);

Route::post("/change-password", [ClientController::class, "change_password"]);
Route::post("/save-profile", [ClientController::class, "save_profile"]);
Route::post("/logout", [ClientController::class, "logout"]);
Route::post("/me", [ClientController::class, "me"]);

Route::group([
    "middleware" => ["auth:sanctum"]
], function () {
    Route::post("/messages/fetch", [MessagesController::class, "fetch"]);
    Route::post("/messages/send", [MessagesController::class, "send"]);

    Route::post("/admin/tasks/delete", [TaskController::class, "destroy"]);
    Route::post("/admin/tasks/update", [TaskController::class, "update"]);
    Route::post("/admin/tasks/fetch", [TaskController::class, "fetch_admin"]);
    Route::post("/admin/tasks/add", [TaskController::class, "store"]);

    Route::post("/admin/projects/delete", [ProjectController::class, "destroy"]);
    Route::post("/admin/projects/update", [ProjectController::class, "update"]);
    Route::post("/admin/projects/fetch", [ProjectController::class, "fetch_admin"]);
    Route::post("/admin/projects/add", [ProjectController::class, "store"]);

    Route::post("/admin/clients/delete", [ClientController::class, "destroy"]);
    Route::post("/admin/clients/update", [ClientController::class, "update"]);
    Route::post("/admin/clients/fetch", [ClientController::class, "fetch"]);
    Route::post("/admin/clients/add", [ClientController::class, "store"]);

    Route::post("/admin/send-message", [AdminController::class, "send_message"]);
    Route::post("/admin/fetch-messages", [AdminController::class, "fetch_messages"]);
    Route::post("/admin/fetch-contacts", [AdminController::class, "fetch_contacts"]);
    Route::post("/admin/users/add", [AdminController::class, "add_user"]);
    Route::post("/admin/users/change-password", [AdminController::class, "change_user_password"]);
    Route::post("/admin/users/delete", [AdminController::class, "delete_user"]);
    Route::post("/admin/users/update", [AdminController::class, "update_user"]);
    Route::post("/admin/users/fetch/{id}", [AdminController::class, "fetch_single_user"]);
    Route::post("/admin/users/fetch", [AdminController::class, "fetch_users"]);
    Route::post("/admin/fetch-settings", [AdminController::class, "fetch_settings"]);
    Route::post("/admin/save-settings", [AdminController::class, "save_settings"]);

    Route::post("/admin/me", [AdminController::class, "me"]);
});
